#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
CS vs Scn
Created on Tue Feb 21 13:17:26 2017

@author: ssim
"""

import numpy as np
import matplotlib.pyplot as plt
plt.style.use('ggplot')
import mpl_toolkits.basemap.pyproj as pyproj
from scipy.interpolate import LinearNDInterpolator as LNDI
import scipy.spatial as ss 
from mpl_toolkits.basemap import Basemap
import matplotlib.path as mplPath
import glob
import sys
import warnings
from datetime import datetime as ddt
import datetime
import time 
import shutil
import os



if not sys.warnoptions:
    warnings.simplefilter("ignore")

def toYearFraction(date):
    def sinceEpoch(date): # returns seconds since epoch
        return time.mktime(date.timetuple())
    s = sinceEpoch

    year = date.year
    startOfThisYear = ddt(year=year, month=1, day=1)
    startOfNextYear = ddt(year=year+1, month=1, day=1)

    yearElapsed = s(date) - s(startOfThisYear)
    yearDuration = s(startOfNextYear) - s(startOfThisYear)
    fraction = yearElapsed/yearDuration
    return date.year + fraction


fig_width_cm = 21                         # A4 page
fig_height_cm = 29.7
inches_per_cm = 1 / 2.58               # Convert cm to inches
fig_width = fig_width_cm * inches_per_cm # width in inches
fig_height = fig_height_cm * inches_per_cm       # height in inches
fig_size = [fig_height, fig_width]

# create polar stereographic Basemap instance.
llon = 21.
llat = 79.3  
ulon = 28.5
ulat = 80.1
                
map_ax = Basemap(llcrnrlon=llon,llcrnrlat=llat,urcrnrlon=ulon,urcrnrlat=ulat, epsg=3995)
        

parallels = np.arange(0.,90,0.5)
meridians = np.arange(0.,360.,2)

# Define standad projection for the polar stenographic projection
scn = np.genfromtxt('data/ALS_2016_Aust.ave', names=['time', 'lat', 'lon', 'h'])
diny = 366.
scn['time']= scn['time']*2016.+106.5/diny

Pollat0  = 90.
Pollon0  = np.nanmean(scn['lon'])
Pollatts = np.nanmean(scn['lat'])
ell = 'WGS84'
PolProj=pyproj.Proj(proj='stere', ellps=ell, lat_0=Pollat0, lon_0=Pollon0, lat_ts=Pollatts,x_0=0,y_0=0,k_0=1.0)
    

xscn,yscn = PolProj(scn['lon'],scn['lat'])
tree =ss.cKDTree(zip(xscn,yscn))
elv = scn['h']
Fscn = LNDI((xscn[np.isnan(elv)==0],yscn[np.isnan(elv)==0]), elv[np.isnan(elv)==0] )


poly1 = np.genfromtxt('data/Aust_poly.txt', names=['lat', 'lon'])
poly2 = np.genfromtxt('data/Aust_poly2.txt', names=['lat', 'lon'])
poly3 = np.genfromtxt('data/Aust_poly3.txt', names=['lat', 'lon'])


print "================ NB !!!! ================"
print "If you would like to add your own data,"  
print "We suggest that you provide data for the full aprill 2016 Austfonna solution of your cryosat processing"
print "Please prepare the following columns"
print "Column 1 : Decimal year"
print "Column 2 : latitude [deg N]"
print "Column 3 : longitude [deg E] (negative for west)"
print "Column 4 : Surface elevation [m]"
print "Column 5 : nadir longitude [deg E] (negative for west)"
print "Column 6 : nadir latitude [deg N]"
print "========================================="   

filelist =[] 
newdatabool = 0     
newdataflag = raw_input('Are the new dataset ready and do you whant to use it (y/n) :')
if newdataflag == 'y':
    newdatabool = 1
    while newdataflag =='y':
        newpath = raw_input('Please provide the path to the ascii data :')
        newname = raw_input('Please provide a name for this dataset:')
        shutil.copyfile(newpath, 'data/'+newname+'_CSSIN2016-4p.txt') 
        
        filelist = np.append(filelist, 'data/'+newname+'_CSSIN2016-4p.txt')
        print "  This dataset will henceafter be named: "
        print "                 "+newname+"                 "
        print "========================================="  
            
        newdataflag = raw_input('Do you want to add another dataset Are the new dataset ready and do you whant to use it (y/n) :')
    
else:
    print "========================================="  
    print " You have choosen not to add new data    "
    print " Showing the original calval results     "
    print "========================================="  
    

CSnames = map(lambda each:each.strip("data/")[:-16], glob.glob('data/*CSSIN2016-4p.txt'))
CSnames = np.array(CSnames)
CSnames = CSnames[np.random.permutation(len(CSnames))]

for ppii in range(1,4):
    print "=========================================" 
    if ppii == 1:
        print '  Evaluating the full area of ALS coverage (poly 1)'         
    elif ppii == 2:
        print 'Evaluating the central area of ALS coverage (poly 2)'     
    elif ppii == 3:
        print 'Evaluating the lower part of basin 3 with ALS coverage (poly 3)'    
    print "=========================================" 
    for dirs  in CSnames:
        cs = np.genfromtxt('data/'+dirs+'_CSSIN2016-4p.txt', names=['time','lat', 'lon', 'h', 'nlon', 'nlat'])
        xcs,ycs = PolProj(cs['lon'],cs['lat'])
        
        exec('poly=poly'+str(ppii))
        xpoly,ypoly = PolProj(poly['lon'],poly['lat'])
        p = []
        for ii in range(len(xpoly)):
            p.append([xpoly[ii], ypoly[ii]])  
        Path = mplPath.Path(p)
        polymask=Path.contains_points(zip(xcs,ycs))
        
        _ , idscn = tree.query(np.column_stack((xcs,ycs)), k=1)
        dist = np.sqrt((xcs-xscn[idscn])**2+(ycs-yscn[idscn])**2)
        dtime = (cs['time']-scn['time'][idscn])*diny
        
        SCNelv = Fscn(xcs,ycs)
        diff = cs['h']-SCNelv
                 
        med = np.median(diff[np.isnan(diff)==0])
        msk = (np.isnan(diff)==0) & polymask
        exec(dirs+ '=np.column_stack((cs["nlat"][msk], cs["nlon"][msk], diff[msk],cs["lon"][msk],cs["lat"][msk],cs["time"][msk]))')
        exec(dirs+ 'full=np.column_stack((cs["nlat"], cs["nlon"], diff,cs["lon"],cs["lat"],cs["time"]))')
    
    if newdataflag == 'y':
        CSnames = np.append(CSnames, 'Newdata')
        
    
    exec('A = np.array('+CSnames[0]+'[:,:2]).round(decimals=4)')
    for nn in CSnames[1:]:
        exec('B = np.array('+nn+'[:,:2]).round(decimals=4)')
        nrows, ncols = A.shape
        dtype={'names':['f{}'.format(i) for i in range(ncols)],
               'formats':ncols * [A.dtype]}    
        C = np.intersect1d(A.view(dtype), B.view(dtype))    
        # This last bit is optional if you're okay with "C" being a structured array...
        C = C.view(A.dtype).reshape(-1, ncols)
        A=C
    
    count = 0
    bins = np.concatenate((np.arange(-50,51,1), np.arange(75,275,25)))
    for dirs  in CSnames:
        if dirs == 'JPL':
            txt = 'JPL'
        elif dirs == 'ESA_L2iC':
            txt = 'ESA'
        elif dirs == 'LARS_ST10':
            txt = 'LARS'# (ST10)'
        elif dirs == 'UoO':
            txt = 'UoO'
        elif dirs == 'UoOroll':
            txt = 'UoO'# (Roll)'
        elif dirs == 'AWI':
            txt = 'AWI'
        elif dirs == 'AWIDEM':
            txt = 'AWI (DEM)'
        elif dirs == 'LARS_NPP50':
            txt = 'LARS (NPP50)'
        else:
            txt = dirs
        cc = map(tuple, C)
        exec('tt = map(tuple, np.array('+dirs+'[:,:2]).round(decimals=4))')
        cc_ind = dict((k,i) for i,k in enumerate(cc))
        tt_ind = dict((k,i) for i,k in enumerate(tt))
        inter = set(cc_ind).intersection(set(tt_ind))
        cc_indices = [ cc_ind[x] for x in inter ]
        tt_indices = [ tt_ind[x] for x in inter ]
        exec('tmpdiff = '+dirs)
        tmplist = tmpdiff[:,2]
        plt.figure(figsize = fig_size)
        #plt.subplot(221)
        ax1=plt.gca()
        plt.hist(tmplist, bins=bins, color='r')
        plt.hist(tmplist[tmplist>50.], bins=bins, color='b')
        plt.title('All points')
        ax1.set_ylim([0, 200])
        ax1.set_xlim([-50, 250 ])
        plt.savefig('ALSvs'+dirs+'Hist_poly'+str(ppii)+'.png')
        
        print "=============   "+ dirs +" ==================="  
        print "           Observations,  Median,  Mean,   Std."    
        print " All obs.      "+ str(len(tmplist))+",        "+str(np.median(tmplist).round(decimals=2))+", "+ str(np.mean(tmplist).round(decimals=2))\
                +", "+str(np.std(tmplist).round(decimals=2))
        tmplist = tmpdiff[tt_indices,2] 
        if dirs == 'AWI':
            AWI_out = tmplist 
        elif dirs == 'AWIDEM':
            AWIDEM_out = tmplist 
        plt.figure(figsize = fig_size)
        #plt.subplot(223)
        ax1=plt.gca()
        plt.hist(tmplist, bins=bins, color='r')
        plt.hist(tmplist[tmplist>50.], bins=bins, color='b')
        plt.title('Only com. Nadir')
        ax1.annotate(txt, xy=(75,160), fontsize=40)
        ax1.set_ylim([0, 200])
        ax1.set_xlim([-50, 250 ])
        for item in ([ax1.title, ax1.xaxis.label, ax1.yaxis.label] + ax1.get_xticklabels() + ax1.get_yticklabels()):
            item.set_fontsize(25)
        plt.savefig('ALSvs'+dirs+'HistNadir_poly'+str(ppii)+'.png')
        print " Only com.     "+ str(len(tmplist))+",        "+str(np.median(tmplist).round(decimals=2))+", "+ str(np.mean(tmplist).round(decimals=2))\
                +", "+str(np.std(tmplist).round(decimals=2)) 
            
        #if count > 3:
        plt.figure(figsize = fig_size)
        #plt.subplot(122)
        #map_ax.etopo()
        map_ax.arcgisimage(server='http://server.arcgisonline.com/ArcGIS', service='World_Imagery', xpixels = 1000, verbose= True)
        map_ax.drawparallels(parallels,labels=[1,0,0,0],fontsize=18,latmax=88)
        map_ax.drawmeridians(meridians,labels=[0,0,0,1],fontsize=18,latmax=88)
        map_ax.plot(np.append(poly['lon'],poly['lon'][0]),np.append(poly['lat'],poly['lat'][0]), 'k',lw=2, latlon=True)
        exec('map_ax.scatter('+dirs+'full[:,3], '+dirs+'full[:,4], s=1, c="k", latlon=True, marker="o",lw=0, label = "Cryosat")') 
        map_ax.scatter(scn['lon'], scn['lat'], s=1, c='g', latlon=True, marker='o',lw=0, label = 'In situ') 
        exec('csm = map_ax.scatter('+dirs+'[:,3],'+dirs+'[:,4], s=30, c='+dirs+'[:,2], latlon=True, cmap="gnuplot",  marker="o", lw = 0, vmin=-5, vmax=25, label = "Difference [m]")') 
        cbar = map_ax.colorbar(csm)
        cbar.set_label('Elevation difference (ALS vs. '+dirs+') [m]',fontsize=18)
        plt.savefig('ALSvs'+dirs+'Map_poly'+str(ppii)+'.png')
        count +=1

if newdatabool == 1:
    for ff in filelist:
        os.remove(ff)

